# babel-plugin-react-flow-props-to-prop-types

> Convert Flow React props annotation to PropTypes

- Supports most Flow types
- Maintains comments
- Works across modules (can import types)

## Example

In:

```js
class MyComponent extends React.Component {
  props: {
    className: string
  };

  // ...
}
```

Out:

```js
class MyComponent extends React.Component {
  props: {
    className: string
  };
  static propTypes = {
    className: PropTypes.string.isRequired
  };

  // ...
}
```

## Installation

```sh
$ npm install react-flow-props-to-prop-types
```

## Usage

### Via `.babelrc` (Recommended)

**.babelrc**

```js
{
  "plugins": [
    ["react-flow-props-to-prop-types", { /* options */ }]
  ]
}
```

### Via CLI

```sh
$ babel --plugins react-flow-props-to-prop-types script.js
```

### Via Node API

```javascript
require("babel-core").transform("code", {
  plugins: [
    ["react-flow-props-to-prop-types", { /* options */ }]
  ]
});
```

## Options

### `resolveOpts` (optional)

Passed through to [node-resolve](https://github.com/substack/node-resolve)
